// Enable and select radio type attached
#define MY_RADIO_NRF24
#define MY_DEBUG
//#define MY_PARENT_NODE_ID 0
//#define MY_PARENT_NODE_IS_STATIC
#include <MySensors.h>

#define NUMBER_OF_BUTTONS 3 // Total number of attached relays
#define EXTRA_LED1_ID 11
#define EXTRA_LED2_ID 12
#define EXTRA_LED1_PIN A4
#define EXTRA_LED2_PIN A5

int ledPins[] = {5,6,8};
byte buttonPins[] = {A1,A2,A6};
byte relayPins[] = {3,4,7};
bool buttonStates[] = {false, false, false};
bool touchStates[] = {false, false, false};
long buttonLastChange[] = {0,0,0};


byte switchOnCommands[] = {8,24,136};
byte switchOffCommands[] = {16,128,144};
#define RESET_PORT_COMMAND 103


MyMessage msg(1, V_TRIPPED);


void before()
{
  // initialize led, relays pins as outputs and buttons as inputs
  for (int i = 0; i < NUMBER_OF_BUTTONS; i++) {
    pinMode(ledPins[i], OUTPUT);
    digitalWrite(ledPins[i], buttonStates[i]);
    pinMode(buttonPins[i], INPUT);
    pinMode(relayPins[i], OUTPUT);
  }
  // initialize extra leds as outputs
  pinMode(EXTRA_LED1_PIN, OUTPUT);
  pinMode(EXTRA_LED2_PIN, OUTPUT);
}

void presentation()
{
  // Send the sketch version information to the gateway and Controller
  sendSketchInfo("LivoloRelay3", "0.1");

  // Register sensors to gateway
  for (int j = 0; j < NUMBER_OF_BUTTONS; j++) {
    present(j+1, S_BINARY);
  }
  present(EXTRA_LED1_ID, S_BINARY);
  present(EXTRA_LED2_ID, S_BINARY);

//  sleep(500);
//  for (int l = 1; l <= NUMBER_OF_BUTTONS; l++) {
//    switchLight(l, loadState(l));
//  }
//  switchLight(EXTRA_LED1_ID, loadState(EXTRA_LED1_ID));
//  switchLight(EXTRA_LED2_ID, loadState(EXTRA_LED2_ID));
}

void receive(const MyMessage &message)
{
  // We only expect one type of message from controller. But we better check anyway.
  if (message.type==V_STATUS) {
    if (message.sensor<=NUMBER_OF_BUTTONS) {
      switchLight(message.sensor, message.getBool());
      buttonStates[message.sensor-1] = message.getBool();
    }
    else {
            switchLight(message.sensor, message.getBool());
    }
#ifdef MY_DEBUG
    // Write some debug info
    Serial.print("Incoming change for sensor:");
    Serial.print(message.sensor);
    Serial.print(", New status: ");
    Serial.println(message.getBool());
#endif
  }
}

/// Switch off relay
/// relayPosition: position of relay, from 1 to 3
void switchRelay(int relayPosition, bool newStatus) {
   // Physically set the relay status by writing the value directly to the port
   // We first make a "AND" operation with the "reset" value which sets the 3 bits managing the relays to 0
   //  then we make a "OR" with the byte setting the right bits for the relays we want to switch on/off
#ifdef MY_DEBUG
   Serial.print("PORTD BEFORE SWITCH :");
   Serial.println(PORTD,BIN);
#endif
   PORTD = (PORTD & RESET_PORT_COMMAND) | (newStatus?switchOnCommands[relayPosition-1]:switchOffCommands[relayPosition-1]);
#ifdef MY_DEBUG
   Serial.print("PORTD AFTER SWITCH :");
   Serial.println(PORTD,BIN);
#endif
}

/// Change status of "switch" or extra led
void switchLight(int sensorID, bool newStatus) {
    // if this is a physical relay, we need to switch on the relay
    if (sensorID<=NUMBER_OF_BUTTONS) {
      // Switches the relay    
      switchRelay(sensorID, newStatus);
      // Update led
      digitalWrite(ledPins[sensorID-1], newStatus);
    }
    if (sensorID == EXTRA_LED1_ID) {
      // set led status 
      digitalWrite(EXTRA_LED1_PIN, newStatus);
    }
    if (sensorID == EXTRA_LED2_ID) {
      // set led status 
      digitalWrite(EXTRA_LED2_PIN, newStatus);
    }
    // Store state in eeprom
    saveState(sensorID, newStatus);    
}


void loop() {
  // put your main code here, to run repeatedly:
  for (int i = 0; i < NUMBER_OF_BUTTONS; i++) {
    // Test input pins, connected to analog inputs so we use analog read, 
    //  if > to half of max value we consider it a HIGH value, else LOW
    bool pinValue = (analogRead(buttonPins[i]) > 512);
    if (pinValue != touchStates[i]) {
      buttonLastChange[i] = millis();
      touchStates[i] = pinValue;
      if (pinValue) {
        #ifdef MY_DEBUG
           Serial.print("Touch on button ");
           Serial.println(i+1);
        #endif
        // if user just pressed on a button, we change the status of the button/switch
        buttonStates[i] = !buttonStates[i];
        // then set the relay
        switchLight(i+1, buttonStates[i]);
        // and finally as we made a local change of the switch position, send value to gateway/controller :
        send(msg.setSensor(i+1).set(buttonStates[i]));
      }
      else {
        #ifdef MY_DEBUG
           Serial.print("Release of button ");
           Serial.println(i+1);
        #endif
      }
    }    
  }
  wait(10);
}
